
Imports Microsoft.DirectX
Imports Microsoft.DirectX.Direct3D
Imports Microsoft.DirectX.DirectInput

Public Class BaseModel
    Private mesh As mesh
    Private meshTexture() As Texture
    Private meshMaterial() As Material
    Private localMatrix As Matrix
    Private pos As Vector3
    Private Vector As Vector3 = New Vector3(0.0F, 0.0F, 0.0F)
    Private m_speed As Single = 0.0F
    Private m_direction As Single = -Math.PI / 2
    Private deltaV As Single = 0.001F
    Private Const max_speed As Single = 0.5F
    Private Const BrakeConst As Single = 1.09      '1傫ȂƂƂ܂܂

    Friend Sub New()
        mesh = Nothing
        meshTexture = Nothing
        meshMaterial = Nothing
        localMatrix = New Matrix()
        localMatrix = Matrix.Identity
    End Sub

    Friend Property Matrix() As Matrix
        Get
            Return localMatrix
        End Get
        Set(ByVal Value As Matrix)
            localMatrix = Value
        End Set
    End Property

    Friend Property Position() As Vector3
        Get
            Return pos
        End Get
        Set(ByVal Value As Vector3)
            pos = Value
        End Set
    End Property

    Friend Property ForwardVector() As Vector3
        Get
            Return Vector
        End Get
        Set(ByVal Value As Vector3)
            Vector = Value
        End Set
    End Property

    Friend Sub Accel()
        m_speed += deltaV
        If m_speed > max_speed Then m_speed = max_speed
    End Sub

    Friend Sub Brake()
        m_speed /= BrakeConst
        If Math.Abs(m_speed) < deltaV Then m_speed = 0
    End Sub

    Friend Sub Back()
        m_speed -= deltaV
        If m_speed < -max_speed Then m_speed = -max_speed
    End Sub

    Friend Sub AccelFree()
        If m_speed <> 0 Then m_speed += -deltaV / 5 * (Math.Abs(m_speed) / m_speed)
    End Sub

    Friend Sub Move()
        pos.X += m_speed * Math.Cos(m_direction)
        pos.Y += 0
        pos.Z += m_speed * Math.Sin(m_direction)

        'pos.X += Vector.X
        'pos.Y += Vector.Y
        'pos.Z += Vector.Z

        localMatrix = Matrix.Translation(pos)
    End Sub

    Friend Function loadMesh(ByRef dev As Direct3D.Device, ByVal filename As String) As Boolean
        Dim materials As ExtendedMaterial() = Nothing
        Try
            mesh = mesh.FromFile(filename, MeshFlags.SystemMemory, dev, materials)

            If meshTexture Is Nothing Then
                meshTexture = New Texture(materials.Length) {}
                meshMaterial = New Direct3D.Material(materials.Length) {}
                Dim i As Integer
                For i = 0 To materials.Length - 1
                    meshMaterial(i) = materials(i).Material3D
                    meshMaterial(i).Ambient = meshMaterial(i).Diffuse

                    If materials(i).TextureFilename <> "" Then
                        meshTexture(i) = TextureLoader.FromFile(dev, materials(i).TextureFilename)
                    End If
                Next i
            End If
        Catch
            Return False
        End Try

        Return True
    End Function

    Friend Function DrawModel(ByRef dev As Direct3D.Device) As Boolean
        Dim i As Integer

        dev.Transform.World = localMatrix
        For i = 0 To meshMaterial.Length - 1
            dev.Material = meshMaterial(i)
            dev.SetTexture(0, meshTexture(i))
            mesh.DrawSubset(i)
        Next
    End Function


End Class
